@@ -1,10 +1,9 @@
 package Spreadsheet::ParseXLSX;
-BEGIN {
-  $Spreadsheet::ParseXLSX::AUTHORITY = 'cpan:DOY';
-}
-$Spreadsheet::ParseXLSX::VERSION = '0.16';
+our $AUTHORITY = 'cpan:DOY';
+$Spreadsheet::ParseXLSX::VERSION = '0.17';
 use strict;
 use warnings;
+use 5.010;
 # ABSTRACT: parse XLSX files
 
 use Archive::Zip;
@@ -60,7 +59,8 @@ sub _parse_workbook {
                                  : (""));
     }
 
-    $workbook->{Flag1904} = $properties && $properties->att('date1904') ? 1 : 0;
+    $workbook->{Flg1904} = $self->_xml_boolean($properties->att('date1904'))
+        if $properties;
 
     $workbook->{FmtClass} = $formatter || Spreadsheet::ParseExcel::FmtDefault->new;
 
@@ -99,7 +99,7 @@ sub _parse_workbook {
     $workbook->{SheetCount} = scalar(@sheets);
 
     my ($node) = $files->{workbook}->find_nodes('//workbookView');
-    my $selected = $node->att('activeTab');
+    my $selected = $node ? $node->att('activeTab') : undef;
     $workbook->{SelectedSheet} = defined($selected) ? 0+$selected : 0;
 
     return $workbook;
@@ -136,8 +136,45 @@ sub _parse_sheet {
 
                 $sheet->{MinRow} = $rmin;
                 $sheet->{MinCol} = $cmin;
-                $sheet->{MaxRow} = $rmax;
-                $sheet->{MaxCol} = $cmax;
+                $sheet->{MaxRow} = $rmax ? $rmax : -1;
+                $sheet->{MaxCol} = $cmax ? $cmax : -1;
+
+                $twig->purge;
+            },
+
+            'headerFooter' => sub {
+                my ($twig, $hf) = @_;
+
+                my ($helem, $felem) = map {
+                    $hf->first_child($_)
+                } qw(oddHeader oddFooter);
+                $sheet->{Header} = $helem->text
+                    if $helem;
+                $sheet->{Footer} = $felem->text
+                    if $felem;
+
+                $twig->purge;
+            },
+
+            'pageMargins' => sub {
+                my ($twig, $margin) = @_;
+                map {
+                    my $key = "\u${_}Margin";
+                    $sheet->{$key} = $margin->att($_) // 0
+                } qw(left right top bottom header footer);
+
+                $twig->purge;
+            },
+
+            'pageSetup' => sub {
+                my ($twig, $setup) = @_;
+                $sheet->{Scale} = $setup->att('scale') // 100;
+                $sheet->{Landscape} = ($setup->att('orientation') // '') ne 'landscape';
+                $sheet->{PaperSize} = $setup->att('paperSize') // 1;
+                $sheet->{PageStart} = $setup->att('firstPageNumber');
+                $sheet->{UsePage} = $self->_xml_boolean($setup->att('useFirstPageNumber'));
+                $sheet->{HorizontalDPI} = $setup->att('horizontalDpi');
+                $sheet->{VerticalDPI} = $setup->att('verticalDpi');
 
                 $twig->purge;
             },
@@ -233,10 +270,18 @@ sub _parse_sheet {
 
                 for my $cell ( $row_elt->children('c') ){
                     my ($row, $col) = $self->_cell_to_row_col($cell->att('r'));
+                    $sheet->{MaxRow} = $row
+                        if $sheet->{MaxRow} < $row;
+                    $sheet->{MaxCol} = $col
+                        if $sheet->{MaxCol} < $col;
                     my $type = $cell->att('t') || 'n';
-                    my $val_xml = $type eq 'inlineStr'
-                        ? $cell->first_child('is')->first_child('t')
-                        : $cell->first_child('v');
+                    my $val_xml;
+                    if ($type ne 'inlineStr') {
+                        $val_xml = $cell->first_child('v');
+                    }
+                    elsif (defined $cell->first_child('is')) {
+                        $val_xml = ($cell->find_nodes('.//t'))[0];
+                    }
                     my $val = $val_xml ? $val_xml->text : undef;
 
                     my $long_type;
@@ -439,29 +484,37 @@ sub _parse_styles {
 
     my @borders = map {
         my $border = $_;
+        my ($ddiag, $udiag) = map {
+            $self->_xml_boolean($border->att($_))
+        } qw(diagonalDown diagonalUp);
+        my %borderstyles = map {
+            my $e = $border->first_child($_);
+            $_ => ($e ? $e->att('style') || 'none' : 'none')
+        } qw(left right top bottom diagonal);
+        my %bordercolors = map {
+            my $e = $border->first_child($_);
+            $_ => ($e ? $e->first_child('color') : undef)
+        } qw(left right top bottom diagonal);
         # XXX specs say "begin" and "end" rather than "left" and "right",
         # but... that's not what seems to be in the file itself (sigh)
         {
             colors => [
                 map {
-                    $self->_color(
-                        $workbook->{Color},
-                        $border->first_child($_)->first_child('color')
-                    )
+                    $self->_color($workbook->{Color}, $bordercolors{$_})
                 } qw(left right top bottom)
             ],
             styles => [
                 map {
-                    $border{$border->first_child($_)->att('style') || 'none'}
+                    $border{$borderstyles{$_}}
                 } qw(left right top bottom)
             ],
             diagonal => [
-                0, # XXX ->att('diagonalDown') and ->att('diagonalUp')
-                0, # XXX ->att('style')
-                $self->_color(
-                    $workbook->{Color},
-                    $border->first_child('diagonal')->first_child('color')
-                ),
+                ( $ddiag &&  $udiag ? 3
+               :  $ddiag && !$udiag ? 2
+               : !$ddiag &&  $udiag ? 1
+               :                      0),
+                $border{$borderstyles{diagonal}},
+                $self->_color($workbook->{Color}, $bordercolors{diagonal}),
             ],
         }
     } $styles->find_nodes('//borders/border');
@@ -509,8 +562,13 @@ sub _parse_styles {
     my @font = map {
         my $vert = $_->first_child('vertAlign');
         my $under = $_->first_child('u');
+        my $heightelem = $_->first_child('sz');
+        # XXX i guess 12 is okay?
+        my $height = 0+($heightelem ? $heightelem->att('val') : 12);
+        my $nameelem = $_->first_child('name');
+        my $name = $nameelem ? $nameelem->att('val') : '';
         Spreadsheet::ParseExcel::Font->new(
-            Height         => 0+$_->first_child('sz')->att('val'),
+            Height         => $height,
             # Attr           => $iAttr,
             # XXX not sure if there's a better way to keep the indexing stuff
             # intact rather than just going straight to #xxxxxx
@@ -542,7 +600,7 @@ sub _parse_styles {
                  :                                  0)
                 : 0
             ),
-            Name           => $_->first_child('name')->att('val'),
+            Name           => $name,
 
             Bold      => $_->has_child('b') ? 1 : 0,
             Italic    => $_->has_child('i') ? 1 : 0,
@@ -555,22 +613,22 @@ sub _parse_styles {
         my $alignment  = $_->first_child('alignment');
         my $protection = $_->first_child('protection');
         Spreadsheet::ParseExcel::Format->new(
-            IgnoreFont         => !$_->att('applyFont'),
-            IgnoreFill         => !$_->att('applyFill'),
-            IgnoreBorder       => !$_->att('applyBorder'),
-            IgnoreAlignment    => !$_->att('applyAlignment'),
-            IgnoreNumberFormat => !$_->att('applyNumberFormat'),
-            IgnoreProtection   => !$_->att('applyProtection'),
+            IgnoreFont         => !$self->_xml_boolean($_->att('applyFont')),
+            IgnoreFill         => !$self->_xml_boolean($_->att('applyFill')),
+            IgnoreBorder       => !$self->_xml_boolean($_->att('applyBorder')),
+            IgnoreAlignment    => !$self->_xml_boolean($_->att('applyAlignment')),
+            IgnoreNumberFormat => !$self->_xml_boolean($_->att('applyNumberFormat')),
+            IgnoreProtection   => !$self->_xml_boolean($_->att('applyProtection')),
 
             FontNo => 0+$_->att('fontId'),
             Font   => $font[$_->att('fontId')],
             FmtIdx => 0+$_->att('numFmtId'),
 
             Lock => $protection && defined $protection->att('locked')
-                ? $protection->att('locked')
+                ? $self->_xml_boolean($protection->att('locked'))
                 : 1,
             Hidden => $protection
-                ? $protection->att('hidden')
+                ? $self->_xml_boolean($protection->att('hidden'))
                 : 0,
             # Style    => $iStyle,
             # Key123   => $i123,
@@ -578,16 +636,18 @@ sub _parse_styles {
                 ? $halign{$alignment->att('horizontal') || 'general'}
                 : 0,
             Wrap => $alignment
-                ? $alignment->att('wrapText')
+                ? $self->_xml_boolean($alignment->att('wrapText'))
                 : 0,
             AlignV => $alignment
                 ? $valign{$alignment->att('vertical') || 'bottom'}
                 : 2,
             # JustLast => $iJustL,
-            # Rotate   => $iRotate,
+            Rotate => $alignment ? $alignment->att('textRotation') : 0,
 
-            # Indent  => $iInd,
-            # Shrink  => $iShrink,
+            Indent => $alignment ? $alignment->att('indent') : 0,
+            Shrink => $alignment
+                ? $self->_xml_boolean($alignment->att('shrinkToFit'))
+                : 0,
             # Merge   => $iMerge,
             # ReadDir => $iReadDir,
 
@@ -726,12 +786,18 @@ sub _cell_to_row_col {
     return ($nrow, $ncol);
 }
 
+sub _xml_boolean {
+    my $self = shift;
+    my ($bool) = @_;
+    return defined($bool) && ($bool eq 'true' || $bool eq '1');
+}
+
 sub _color {
     my $self = shift;
     my ($colors, $color_node, $fill) = @_;
 
     my $color;
-    if ($color_node && !$color_node->att('auto')) {
+    if ($color_node && !$self->_xml_boolean($color_node->att('auto'))) {
         if (defined $color_node->att('indexed')) {
             # see https://rt.cpan.org/Public/Bug/Display.html?id=93065
             if ($fill && $color_node->att('indexed') == 64) {
@@ -747,7 +813,13 @@ sub _color {
             $color = '#' . substr($color_node->att('rgb'), 2, 6);
         }
         elsif (defined $color_node->att('theme')) {
-            $color = '#' . $colors->[$color_node->att('theme')];
+            my $theme = $colors->[$color_node->att('theme')];
+            if (defined $theme) {
+                $color = "#$theme";
+            }
+            else {
+                return;
+            }
         }
 
         $color = $self->_apply_tint($color, $color_node->att('tint'))
@@ -789,7 +861,7 @@ Spreadsheet::ParseXLSX - parse XLSX files
 
 =head1 VERSION
 
-version 0.16
+version 0.17
 
 =head1 SYNOPSIS
 
@@ -928,7 +1000,7 @@ Jesse Luehrs <doy@tozt.net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Jesse Luehrs.
+This software is Copyright (c) 2015 by Jesse Luehrs.
 
 This is free software, licensed under:
 
